@@ -1,55 +1,17 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
- * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #ifndef APR_POOLS_H
@@ -132,7 +94,7 @@ typedef struct apr_pool_t apr_pool_t;
  * <pre>
  * | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
  * ---------------------------------
- * |   |   |   |   |   |   |   | x |  General debug code enabled (usefull in
+ * |   |   |   |   |   |   |   | x |  General debug code enabled (useful in
  *                                    combination with --with-efence).
  *
  * |   |   |   |   |   |   | x |   |  Verbose output on stderr (report
@@ -162,7 +124,8 @@ typedef struct apr_pool_t apr_pool_t;
  * </pre>
  */
 #if defined(APR_POOL_DEBUG)
-#if (APR_POOL_DEBUG != 0) && (APR_POOL_DEBUG - 0 == 0)
+/* If APR_POOL_DEBUG is blank, we get 1; if it is a number, we get -1. */
+#if (APR_POOL_DEBUG - APR_POOL_DEBUG -1 == 1)
 #undef APR_POOL_DEBUG
 #define APR_POOL_DEBUG 1
 #endif
@@ -275,17 +238,6 @@ APR_DECLARE(apr_status_t) apr_pool_create(apr_pool_t **newpool,
 #endif
 #endif
 
-/** @deprecated @see apr_pool_create_ex */
-#if APR_POOL_DEBUG
-#define apr_pool_sub_make(newpool, parent, abort_fn) \
-    (void)apr_pool_create_ex_debug(newpool, parent, abort_fn, \
-                                   NULL, \
-                                   APR_POOL__FILE_LINE__)
-#else
-#define apr_pool_sub_make(newpool, parent, abort_fn) \
-    (void)apr_pool_create_ex(newpool, parent, abort_fn, NULL)
-#endif
-
 /**
  * Find the pools allocator
  * @param pool The pool to get the allocator from.
@@ -425,10 +377,6 @@ APR_DECLARE(void *) apr_pcalloc_debug(apr_pool_t *p, apr_size_t size,
 APR_DECLARE(void) apr_pool_abort_set(apr_abortfunc_t abortfunc,
                                      apr_pool_t *pool);
 
-/** @deprecated @see apr_pool_abort_set */
-APR_DECLARE(void) apr_pool_set_abort(apr_abortfunc_t abortfunc,
-                                     apr_pool_t *pool);
-
 /**
  * Get the abort function associated with the specified pool.
  * @param pool The pool for retrieving the abort function.
@@ -436,9 +384,6 @@ APR_DECLARE(void) apr_pool_set_abort(apr_abortfunc_t abortfunc,
  */
 APR_DECLARE(apr_abortfunc_t) apr_pool_abort_get(apr_pool_t *pool);
 
-/** @deprecated @see apr_pool_abort_get */
-APR_DECLARE(apr_abortfunc_t) apr_pool_get_abort(apr_pool_t *pool);
-
 /**
  * Get the parent pool of the specified pool.
  * @param pool The pool for retrieving the parent pool.
@@ -446,15 +391,16 @@ APR_DECLARE(apr_abortfunc_t) apr_pool_get_abort(apr_pool_t *pool);
  */
 APR_DECLARE(apr_pool_t *) apr_pool_parent_get(apr_pool_t *pool);
 
-/** @deprecated @see apr_pool_parent_get */
-APR_DECLARE(apr_pool_t *) apr_pool_get_parent(apr_pool_t *pool);
-
 /**
- * Determine if pool a is an ancestor of pool b
+ * Determine if pool a is an ancestor of pool b.
  * @param a The pool to search
  * @param b The pool to search for
  * @return True if a is an ancestor of b, NULL is considered an ancestor
  *         of all pools.
+ * @remark if compiled with APR_POOL_DEBUG, this function will also
+ * return true if A is a pool which has been guaranteed by the caller
+ * (using apr_pool_join) to have a lifetime at least as long as some
+ * ancestor of pool B.
  */
 APR_DECLARE(int) apr_pool_is_ancestor(apr_pool_t *a, apr_pool_t *b);
 
@@ -481,10 +427,13 @@ APR_DECLARE(void) apr_pool_tag(apr_pool_t *pool, const char *tag);
  *
  *      Users of APR must take EXTREME care when choosing a key to
  *      use for their data.  It is possible to accidentally overwrite
- *      data by choosing a key that another part of the program is using
- *      It is advised that steps are taken to ensure that a unique
- *      key is used at all times.
- * @bug Specify how to ensure this uniqueness!
+ *      data by choosing a key that another part of the program is using.
+ *      Therefore it is advised that steps are taken to ensure that unique
+ *      keys are used for all of the userdata objects in a particular pool
+ *      (the same key in two different pools or a pool and one of its
+ *      subpools is okay) at all times.  Careful namespace prefixing of
+ *      key names is a typical way to help ensure this uniqueness.
+ *
  */
 APR_DECLARE(apr_status_t) apr_pool_userdata_set(
     const void *data,
@@ -527,11 +476,18 @@ APR_DECLARE(apr_status_t) apr_pool_userdata_get(void **data, const char *key,
                                                 apr_pool_t *pool);
 
 
-/*
- * Cleanup
+/**
+ * @defgroup PoolCleanup  Pool Cleanup Functions
  *
  * Cleanups are performed in the reverse order they were registered.  That is:
- * Last In, First Out.
+ * Last In, First Out.  A cleanup function can safely allocate memory from
+ * the pool that is being cleaned up. It can also safely register additional
+ * cleanups which will be run LIFO, directly after the current cleanup
+ * terminates.  Cleanups have to take caution in calling functions that
+ * create subpools. Subpools, created during cleanup will NOT automatically
+ * be cleaned up.  In other words, cleanups are to clean up after themselves.
+ *
+ * @{
  */
 
 /**
@@ -540,8 +496,8 @@ APR_DECLARE(apr_status_t) apr_pool_userdata_get(void **data, const char *key,
  * @param data The data to pass to the cleanup function.
  * @param plain_cleanup The function to call when the pool is cleared
  *                      or destroyed
- * @param child_cleanup The function to call when a child process is being
- *                      shutdown - this function is called in the child, obviously!
+ * @param child_cleanup The function to call when a child process is about
+ *                      to exec - this function is called in the child, obviously!
  */
 APR_DECLARE(void) apr_pool_cleanup_register(
     apr_pool_t *p,
@@ -550,9 +506,13 @@ APR_DECLARE(void) apr_pool_cleanup_register(
     apr_status_t (*child_cleanup)(void *));
 
 /**
- * Remove a previously registered cleanup function
- * @param p The pool remove the cleanup from
- * @param data The data to remove from cleanup
+ * Remove a previously registered cleanup function.
+ * 
+ * The cleanup most recently registered with @a p having the same values of
+ * @a data and @a cleanup will be removed.
+ *
+ * @param p The pool to remove the cleanup from
+ * @param data The data of the registered cleanup
  * @param cleanup The function to remove from cleanup
  * @remarks For some strange reason only the plain_cleanup is handled by this
  *          function
@@ -561,7 +521,12 @@ APR_DECLARE(void) apr_pool_cleanup_kill(apr_pool_t *p, const void *data,
                                         apr_status_t (*cleanup)(void *));
 
 /**
- * Replace the child cleanup of a previously registered cleanup
+ * Replace the child cleanup function of a previously registered cleanup.
+ * 
+ * The cleanup most recently registered with @a p having the same values of
+ * @a data and @a plain_cleanup will have the registered child cleanup
+ * function replaced with @a child_cleanup.
+ *
  * @param p The pool of the registered cleanup
  * @param data The data of the registered cleanup
  * @param plain_cleanup The plain cleanup function of the registered cleanup
@@ -574,9 +539,13 @@ APR_DECLARE(void) apr_pool_child_cleanup_set(
     apr_status_t (*child_cleanup)(void *));
 
 /**
- * Run the specified cleanup function immediately and unregister it. Use
- * @a data instead of the data that was registered with the cleanup.
- * @param p The pool remove the cleanup from
+ * Run the specified cleanup function immediately and unregister it.
+ *
+ * The cleanup most recently registered with @a p having the same values of
+ * @a data and @a cleanup will be removed and @a cleanup will be called
+ * with @a data as the argument.
+ *
+ * @param p The pool to remove the cleanup from
  * @param data The data to remove from cleanup
  * @param cleanup The function to remove from cleanup
  */
@@ -586,20 +555,23 @@ APR_DECLARE(apr_status_t) apr_pool_cleanup_run(
     apr_status_t (*cleanup)(void *));
 
 /**
- * An empty cleanup function
- * @param data The data to cleanup
+ * An empty cleanup function.
+ * 
+ * Passed to apr_pool_cleanup_register() when no cleanup is required.
+ *
+ * @param data The data to cleanup, will not be used by this function.
  */
 APR_DECLARE_NONSTD(apr_status_t) apr_pool_cleanup_null(void *data);
 
-/* Preparing for exec() --- close files, etc., but *don't* flush I/O
- * buffers, *don't* wait for subprocesses, and *don't* free any memory.
- */
 /**
- * Run all of the child_cleanups, so that any unnecessary files are
- * closed because we are about to exec a new program
+ * Run all registered child cleanups, in preparation for an exec()
+ * call in a forked child -- close files, etc., but *don't* flush I/O
+ * buffers, *don't* wait for subprocesses, and *don't* free any
+ * memory.
  */
 APR_DECLARE(void) apr_pool_cleanup_for_exec(void);
 
+/** @} */
 
 /**
  * @defgroup PoolDebug Pool Debugging functions.
@@ -630,16 +602,14 @@ APR_DECLARE(void) apr_pool_cleanup_for_exec(void);
  * structures.
  *
  * However, sometimes this ancestor requirement is inconvenient --
- * sometimes we're forced to create a sub pool (such as through
- * apr_sub_req_lookup_uri), and the sub pool is guaranteed to have
- * the same lifetime as the parent pool.  This is a guarantee implemented
- * by the *caller*, not by the pool code.  That is, the caller guarantees
- * they won't destroy the sub pool individually prior to destroying the
- * parent pool.
+ * sometimes it's necessary to create a sub pool where the sub pool is
+ * guaranteed to have the same lifetime as the parent pool.  This is a
+ * guarantee implemented by the *caller*, not by the pool code.  That
+ * is, the caller guarantees they won't destroy the sub pool
+ * individually prior to destroying the parent pool.
  *
  * In this case the caller must call apr_pool_join() to indicate this
- * guarantee to the APR_POOL_DEBUG code.  There are a few examples spread
- * through the standard modules.
+ * guarantee to the APR_POOL_DEBUG code.
  *
  * These functions are only implemented when #APR_POOL_DEBUG is set.
  *
