@@ -1,4 +1,5 @@
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -75,11 +76,12 @@ apr_status_t apr_socket_timeout_set(apr_socket_t *sock, apr_interval_time_t t)
 {
     apr_status_t stat;
 
-    /* If our timeout is positive or zero and our last timeout was
+    /* If our new timeout is non-negative and our old timeout was
      * negative, then we need to ensure that we are non-blocking.
-     * Conversely, if our timeout is negative and we had a positive
-     * or zero timeout, we must make sure our socket is blocking.
-     * We want to avoid calling fcntl more than necessary on the socket,
+     * Conversely, if our new timeout is negative and we had
+     * non-negative timeout, we must make sure our socket is blocking.
+     * We want to avoid calling fcntl more than necessary on the
+     * socket.
      */
     if (t >= 0 && sock->timeout < 0) {
         if (apr_is_option_set(sock, APR_SO_NONBLOCK) != 1) {
@@ -149,11 +151,8 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
         break;
     case APR_SO_SNDBUF:
 #ifdef SO_SNDBUF
-        if (apr_is_option_set(sock, APR_SO_SNDBUF) != on) {
-            if (setsockopt(sock->socketdes, SOL_SOCKET, SO_SNDBUF, (void *)&on, sizeof(int)) == -1) {
-                return errno;
-            }
-            apr_set_option(sock, APR_SO_SNDBUF, on);
+        if (setsockopt(sock->socketdes, SOL_SOCKET, SO_SNDBUF, (void *)&on, sizeof(int)) == -1) {
+            return errno;
         }
 #else
         return APR_ENOTIMPL;
@@ -161,11 +160,8 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
         break;
     case APR_SO_RCVBUF:
 #ifdef SO_RCVBUF
-        if (apr_is_option_set(sock, APR_SO_RCVBUF) != on) {
-            if (setsockopt(sock->socketdes, SOL_SOCKET, SO_RCVBUF, (void *)&on, sizeof(int)) == -1) {
-                return errno;
-            }
-            apr_set_option(sock, APR_SO_RCVBUF, on);
+        if (setsockopt(sock->socketdes, SOL_SOCKET, SO_RCVBUF, (void *)&on, sizeof(int)) == -1) {
+            return errno;
         }
 #else
         return APR_ENOTIMPL;
@@ -214,6 +210,7 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
 #else
         return APR_ENOTIMPL;
 #endif
+        break;
     case APR_TCP_NODELAY:
 #if defined(TCP_NODELAY)
         if (apr_is_option_set(sock, APR_TCP_NODELAY) != on) {
@@ -246,7 +243,13 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
         break;
     case APR_TCP_NOPUSH:
 #if APR_TCP_NOPUSH_FLAG
+        /* TCP_NODELAY and TCP_CORK are mutually exclusive on Linux
+         * kernels < 2.6; on newer kernels they can be used together
+         * and TCP_CORK takes preference, which is the desired
+         * behaviour.  On older kernels, TCP_NODELAY must be toggled
+         * to "off" whilst TCP_CORK is in effect. */
         if (apr_is_option_set(sock, APR_TCP_NOPUSH) != on) {
+#ifndef HAVE_TCP_NODELAY_WITH_CORK
             int optlevel = IPPROTO_TCP;
             int optname = TCP_NODELAY;
 
@@ -257,11 +260,9 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
             }
 #endif
             /* OK we're going to change some settings here... */
-            /* TCP_NODELAY is mutually exclusive, so do we have it set? */
             if (apr_is_option_set(sock, APR_TCP_NODELAY) == 1 && on) {
-                /* If we want to set NOPUSH then if we have the TCP_NODELAY
-                 * flag set we need to switch it off...
-                 */
+                /* Now toggle TCP_NODELAY to off, if TCP_CORK is being
+                 * turned on: */
                 int tmpflag = 0;
                 if (setsockopt(sock->socketdes, optlevel, optname,
                                (void*)&tmpflag, sizeof(int)) == -1) {
@@ -272,13 +273,19 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
             } else if (on) {
                 apr_set_option(sock, APR_RESET_NODELAY, 0);
             }
+#endif /* HAVE_TCP_NODELAY_WITH_CORK */
+
             /* OK, now we can just set the TCP_NOPUSH flag accordingly...*/
             if (setsockopt(sock->socketdes, IPPROTO_TCP, APR_TCP_NOPUSH_FLAG,
                            (void*)&on, sizeof(int)) == -1) {
                 return errno;
             }
             apr_set_option(sock, APR_TCP_NOPUSH, on);
+#ifndef HAVE_TCP_NODELAY_WITH_CORK
             if (!on && apr_is_option_set(sock, APR_RESET_NODELAY)) {
+                /* Now, if TCP_CORK was just turned off, turn
+                 * TCP_NODELAY back on again if it was earlier toggled
+                 * to off: */
                 int tmpflag = 1;
                 if (setsockopt(sock->socketdes, optlevel, optname,
                                (void*)&tmpflag, sizeof(int)) == -1) {
@@ -287,6 +294,7 @@ apr_status_t apr_socket_opt_set(apr_socket_t *sock,
                 apr_set_option(sock, APR_RESET_NODELAY,0);
                 apr_set_option(sock, APR_TCP_NODELAY, 1);
             }
+#endif /* HAVE_TCP_NODELAY_WITH_CORK */
         }
 #else
         return APR_ENOTIMPL;
@@ -354,7 +362,11 @@ apr_status_t apr_socket_atmark(apr_socket_t *sock, int *atmark)
 
 apr_status_t apr_gethostname(char *buf, apr_int32_t len, apr_pool_t *cont)
 {
-    if (gethostname(buf, len) == -1) {
+#ifdef BEOS_R5
+    if (gethostname(buf, len) == 0) {
+#else
+    if (gethostname(buf, len) != 0) {
+#endif  
         buf[0] = '\0';
         return errno;
     }
