@@ -1,4 +1,5 @@
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -288,7 +289,8 @@ static char *apr_gcvt(double number, int ndigit, char *buf, boolean_e altform)
  */
 #define FIX_PRECISION(adjust, precision, s, s_len)  \
     if (adjust) {                                   \
-        int p = precision < NUM_BUF_SIZE - 1 ? precision : NUM_BUF_SIZE - 1; \
+        apr_size_t p = (precision + 1 < NUM_BUF_SIZE) \
+                     ? precision : NUM_BUF_SIZE - 1;  \
         while (s_len < p)                           \
         {                                           \
             *--s = '0';                             \
@@ -337,7 +339,7 @@ while (width > len)
  */
 static char *conv_10(register wide_int num, register bool_int is_unsigned,
                      register bool_int *is_negative, char *buf_end,
-                     register int *len)
+                     register apr_size_t *len)
 {
     register char *p = buf_end;
     register u_wide_int magnitude;
@@ -384,7 +386,7 @@ static char *conv_10(register wide_int num, register bool_int is_unsigned,
 
 static char *conv_10_quad(widest_int num, register bool_int is_unsigned,
                      register bool_int *is_negative, char *buf_end,
-                     register int *len)
+                     register apr_size_t *len)
 {
     register char *p = buf_end;
     u_widest_int magnitude;
@@ -441,12 +443,12 @@ static char *conv_10_quad(widest_int num, register bool_int is_unsigned,
 
 
 
-static char *conv_in_addr(struct in_addr *ia, char *buf_end, int *len)
+static char *conv_in_addr(struct in_addr *ia, char *buf_end, apr_size_t *len)
 {
     unsigned addr = ntohl(ia->s_addr);
     char *p = buf_end;
     bool_int is_negative;
-    int sub_len;
+    apr_size_t sub_len;
 
     p = conv_10((addr & 0x000000FF)      , TRUE, &is_negative, p, &sub_len);
     *--p = '.';
@@ -462,11 +464,11 @@ static char *conv_in_addr(struct in_addr *ia, char *buf_end, int *len)
 
 
 
-static char *conv_apr_sockaddr(apr_sockaddr_t *sa, char *buf_end, int *len)
+static char *conv_apr_sockaddr(apr_sockaddr_t *sa, char *buf_end, apr_size_t *len)
 {
     char *p = buf_end;
     bool_int is_negative;
-    int sub_len;
+    apr_size_t sub_len;
     char *ipaddr_str;
 
     p = conv_10(sa->port, TRUE, &is_negative, p, &sub_len);
@@ -495,7 +497,7 @@ static char *conv_apr_sockaddr(apr_sockaddr_t *sa, char *buf_end, int *len)
 
 
 #if APR_HAS_THREADS
-static char *conv_os_thread_t(apr_os_thread_t *tid, char *buf_end, int *len)
+static char *conv_os_thread_t(apr_os_thread_t *tid, char *buf_end, apr_size_t *len)
 {
     union {
         apr_os_thread_t tid;
@@ -526,7 +528,7 @@ static char *conv_os_thread_t(apr_os_thread_t *tid, char *buf_end, int *len)
  */
 static char *conv_fp(register char format, register double num,
     boolean_e add_dp, int precision, bool_int *is_negative,
-    char *buf, int *len)
+    char *buf, apr_size_t *len)
 {
     register char *s = buf;
     register char *p;
@@ -580,7 +582,7 @@ static char *conv_fp(register char format, register double num,
 
     if (format != 'f') {
         char temp[EXPONENT_LENGTH];        /* for exponent conversion */
-        int t_len;
+        apr_size_t t_len;
         bool_int exponent_is_negative;
 
         *s++ = format;                /* either e or E */
@@ -624,7 +626,7 @@ static char *conv_fp(register char format, register double num,
  * the number isn't quad size.
  */
 static char *conv_p2(register u_wide_int num, register int nbits,
-                     char format, char *buf_end, register int *len)
+                     char format, char *buf_end, register apr_size_t *len)
 {
     register int mask = (1 << nbits) - 1;
     register char *p = buf_end;
@@ -643,7 +645,7 @@ static char *conv_p2(register u_wide_int num, register int nbits,
 }
 
 static char *conv_p2_quad(u_widest_int num, register int nbits,
-                     char format, char *buf_end, register int *len)
+                     char format, char *buf_end, register apr_size_t *len)
 {
     register int mask = (1 << nbits) - 1;
     register char *p = buf_end;
@@ -664,6 +666,27 @@ static char *conv_p2_quad(u_widest_int num, register int nbits,
     return (p);
 }
 
+#if APR_HAS_THREADS
+static char *conv_os_thread_t_hex(apr_os_thread_t *tid, char *buf_end, apr_size_t *len)
+{
+    union {
+        apr_os_thread_t tid;
+        apr_uint64_t alignme;
+    } u;
+    int is_negative;
+
+    u.tid = *tid;
+    switch(sizeof(u.tid)) {
+    case sizeof(apr_int32_t):
+        return conv_p2(*(apr_uint32_t *)&u.tid, 4, 'x', buf_end, len);
+    case sizeof(apr_int64_t):
+        return conv_p2_quad(*(apr_uint64_t *)&u.tid, 4, 'x', buf_end, len);
+    default:
+        /* not implemented; stick 0 in the buffer */
+        return conv_10(0, TRUE, &is_negative, buf_end, len);
+    }
+}
+#endif
 
 /*
  * Do format conversion placing the output in buffer
@@ -674,14 +697,14 @@ APR_DECLARE(int) apr_vformatter(int (*flush_func)(apr_vformatter_buff_t *),
     register char *sp;
     register char *bep;
     register int cc = 0;
-    register int i;
+    register apr_size_t i;
 
     register char *s = NULL;
     char *q;
-    int s_len;
+    apr_size_t s_len;
 
-    register int min_width = 0;
-    int precision = 0;
+    register apr_size_t min_width = 0;
+    apr_size_t precision = 0;
     enum {
         LEFT, RIGHT
     } adjust;
@@ -761,13 +784,15 @@ APR_DECLARE(int) apr_vformatter(int (*flush_func)(apr_vformatter_buff_t *),
                     adjust_width = YES;
                 }
                 else if (*fmt == '*') {
-                    min_width = va_arg(ap, int);
+                    int v = va_arg(ap, int);
                     fmt++;
                     adjust_width = YES;
-                    if (min_width < 0) {
+                    if (v < 0) {
                         adjust = LEFT;
-                        min_width = -min_width;
+                        min_width = (apr_size_t)(-v);
                     }
+                    else
+                        min_width = (apr_size_t)v;
                 }
                 else
                     adjust_width = NO;
@@ -782,10 +807,9 @@ APR_DECLARE(int) apr_vformatter(int (*flush_func)(apr_vformatter_buff_t *),
                         STR_TO_DEC(fmt, precision);
                     }
                     else if (*fmt == '*') {
-                        precision = va_arg(ap, int);
+                        int v = va_arg(ap, int);
                         fmt++;
-                        if (precision < 0)
-                            precision = 0;
+                        precision = (v < 0) ? 0 : (apr_size_t)v;
                     }
                     else
                         precision = 0;
@@ -1169,6 +1193,31 @@ APR_DECLARE(int) apr_vformatter(int (*flush_func)(apr_vformatter_buff_t *),
 #endif
                     break;
 
+                case 't':
+#if APR_HAS_THREADS
+                {
+                    apr_os_thread_t *tid;
+
+                    tid = va_arg(ap, apr_os_thread_t *);
+                    if (tid != NULL) {
+                        s = conv_os_thread_t_hex(tid, &num_buf[NUM_BUF_SIZE], &s_len);
+                        if (adjust_precision && precision < s_len)
+                            s_len = precision;
+                    }
+                    else {
+                        s = S_NULL;
+                        s_len = S_NULL_LEN;
+                    }
+                    pad_char = ' ';
+                }
+#else
+                    char_buf[0] = '0';
+                    s = &char_buf[0];
+                    s_len = 1;
+                    pad_char = ' ';
+#endif
+                    break;
+
                 case NUL:
                     /* if %p ends the string, oh well ignore it */
                     continue;
